/*
 * Project name
     Time LOCK
 * Copyright
     (c) mikroElektronika, 2011.
 * Revision History
     20110429:
       - initial release (ZR)
     20131022:
       - modified for PIC18F45K22 (MV)
 * Description
       Implementation of Time Lock using mikroElektronika hardware and software solutions.
       Four relays are driven on and off at specific moments of time which can be setup by keypad.
       LCD display is used to setup relays and current time.
              
 * Test configuration:
     MCU:              PIC18F45K22
                       http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf

     Dev.Board:       Ready for PIC Board
                      ac:Ready_for_PIC
                      http://www.mikroe.com/ready/pic/
     Oscillator:      8.0000 MHz  (4xPLL Enabled -> 32.0000 Mhz)
     Ext. Modules:    LCD adapter board
                      ac:LCD_Adapter
                      http://www.mikroe.com/eng/products/view/145/lcd-adapter-board/
                      LCD 2x16 characters
                      ac:LCD_2x16ch
                      http://www.mikroe.com/rs/store/components/displays/
                      Relay4 Board
                      http://www.mikroe.com/eng/products/view/597/relay4-board/
                      ac:Relay4_Board
                      RTC Board
                      http://www.mikroe.com/eng/products/view/136/rtc-board/
                      ac:RTC_Clock
                      Keypad 4x4 Board
                      http://www.mikroe.com/eng/products/view/135/keypad-4x4-board/
                      ac:Keypad_4x4
                      EasyPULL Board
                      http://www.mikroe.com/add-on-boards/various/easypull/
                      ac:EasyPULL_1K
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
     - LCD display should be connected to PORTB
     - EasyPULL Board should be connected to PORTD
     - Keypad 4x4 Board should be connected to EasyPULL Board
     - Relay4 Board should be connected to PORTA
     - RTC Board should be connected to PORTC
     - Terminology: "Lock" <=> "Relay"
     
 * OPERATION:
     - Press "B" to setup current time
     - Press "C" to toggle between setup screens and current time
     - Press "D" to browse through setup screens
     - Press "*" to move cursor to the right
     - Press "#" to move cursor to the left
     - Press "0".."9" to set digits
*/

#include "unglobals.h"

void TurnLockOn(char LockIndex){         // Turn on relay[0..3]
  LockIndex = 1 << LockIndex;
  LATA = PORTA | LockIndex;
}

void TurnLockOff(char LockIndex){        // Turn off relay[0..3]
  LockIndex = ~(1 << LockIndex);
  LATA = PORTA & LockIndex;
}

void CheckLocks(){                        // Check if the time has come
                                         //   to change state of relays
  char i;
  for (i = 0; i < 4; i++)                           // There are four relays
    {
      if (Locks[i].on_hours == hours)        // If hours, minutes and seconds match
        if (Locks[i].on_minutes == minutes)
          if (Locks[i].on_seconds == seconds)
            TurnLockOn(i);                     // Turn on a lock
            
      if (Locks[i].off_hours == hours)        // If hours, minutes and seconds match
        if (Locks[i].off_minutes == minutes)
          if (Locks[i].off_seconds == seconds)
            TurnLockOff(i);                   // Turn off a lock
    }
}

// for testing purposes
// setup relays to turn on/off in sequence of 5 seconds
void TestSetup(){
  Locks[0].on_hours    = 10; Locks[0].off_hours   = 10;
  Locks[0].on_minutes  = 37; Locks[0].off_minutes = 37;
  Locks[0].on_seconds  =  5; Locks[0].off_seconds = 15;

  Locks[1].on_hours    = 10; Locks[1].off_hours   = 10;
  Locks[1].on_minutes  = 37; Locks[1].off_minutes = 37;
  Locks[1].on_seconds  = 10; Locks[1].off_seconds = 20;

  Locks[2].on_hours    = 10; Locks[2].off_hours   = 10;
  Locks[2].on_minutes  = 37; Locks[2].off_minutes = 37;
  Locks[2].on_seconds  = 15; Locks[2].off_seconds = 25;

  Locks[3].on_hours    = 10; Locks[3].off_hours   = 10;
  Locks[3].on_minutes  = 37; Locks[3].off_minutes = 37;
  Locks[3].on_seconds  = 20; Locks[3].off_seconds = 30;
   
  Set_Time(10, 37);  // set time to 10.37h

  CurrentLock = 0;  // Position to the first lock
}

void main(){
  Delay_ms(500);                               // Wait for a while
  Init_All();                                  // Perform initialization
  LATA=0;
  //TestSetup();                                 // Use this for testing, uncomment otherwise
  while (1)                                // }less loop
    {
      Read_Time();                             // Read current time
      if (old_seconds != seconds)           // Check locks each second
        {
          CheckLocks();
          old_seconds = seconds;
        };

      ReadKeypad();                            // Check keypad if there is some action
      if (ShowTime)                          // Show time?
        {
          Locks[CurrentLock].modified = 0xFF; // This will refresh setup screen after time screen is off
          Display_Time();                      // Display current time on LCD
        }
      else
        {
          if (Locks[CurrentLock].modified) // If there was a change in setup
            Display_Alarm_Time();              // Refresh setup screens
        }
    }
}